@@ -25,6 +25,10 @@ def parse_bpmn_anns(bpmn_path: Path):
 
 class InvalidBpmnException(Exception):
     def __init__(self, error_type: str, details: str = None):
+        """
+        :param error_type: makes it possible to group errors by type
+        :param details: can be used to log file specific error details
+        """
         super().__init__(error_type if details is None else f"{error_type}: {details}")
         self.error_type = error_type
         self.details = details
@@ -38,6 +42,9 @@ class BpmnParser:
             excluded_categories: Set[str] = None,
             excluded_label_categories: Set[str] = None,
             link_text_rel_two_way: bool = False,
+            link_pools: bool = True,
+            link_lanes: bool = True,
+            scale_to_ann_width: bool = True
     ):
         """
         :param arrow_min_wh: pad edge bounding boxes so that their w and h is at least arrow_min_wh
@@ -50,6 +57,9 @@ class BpmnParser:
         self.excluded_categories = {} if excluded_categories is None else excluded_categories
         self.excluded_label_categories = {} if excluded_label_categories is None else excluded_label_categories
         self.link_text_rel_two_way = link_text_rel_two_way
+        self.link_pools = link_pools
+        self.link_lanes = link_lanes
+        self.scale_to_ann_width = scale_to_ann_width
 
     def _is_included_ann(self, a: Annotation) -> bool:
         if a.category in self.excluded_categories:
@@ -72,7 +82,14 @@ class BpmnParser:
             raise e
 
         img = yamlu.read_img(img_path)
-        self.scale_anns_to_img_width_(anns, bpmn_path, img)
+
+        arrow_min_wh = self.arrow_min_wh
+        if self.scale_to_ann_width:
+            self.scale_anns_to_img_width_(anns, bpmn_path, img)
+            arrow_min_wh = self.arrow_min_wh * max(img.size) / self.img_max_size_ref
+
+        edge_anns = [a for a in anns if a.category in syntax.BPMNDI_EDGE_CATEGORIES]
+        self.resize_arrows_to_min_wh(edge_anns, arrow_min_wh)
 
         anns = [a for a in anns if self._is_included_ann(a)]
 
@@ -105,31 +122,45 @@ class BpmnParser:
         diagram = root.find("bpmndi:BPMNDiagram", NS_MAP)
         plane = diagram[0]
         shapes = plane.findall("bpmndi:BPMNShape", NS_MAP)
-        shape_anns = []
-        for shape in shapes:
-            model_element = id_to_obj[shape.get("bpmnElement")]
-            if get_ns(model_element) != NS_MODEL:
-                _logger.warning("%s: skipping %s element with custom namespace", bpmn_path, model_element.tag)
-                continue
-            shape_anns += _shape_to_anns(shape, model_element, has_pools=len(collaborations) > 0)
-        id_to_shape_ann = {a.id: a for a in shape_anns if a.category != "label"}
-
         edges = plane.findall("bpmndi:BPMNEdge", NS_MAP)
-        edge_anns = []
-        for edge in edges:
-            model_id = edge.get("bpmnElement")
+        elements = shapes + edges
+
+        associations = []
+        anns = []
+        id_to_ann = {}
+        for element in elements:
+            model_id = element.get("bpmnElement")
             if model_id not in id_to_obj:
-                raise InvalidBpmnException("Missing edge model element", f"{bpmn_path}: {model_id}")
+                raise InvalidBpmnException("Missing model element", f"{bpmn_path}: {model_id}")
             model_element = id_to_obj[model_id]
             if get_ns(model_element) != NS_MODEL:
                 _logger.warning("%s: skipping %s element with custom namespace", bpmn_path, model_element.tag)
                 continue
-            edge_anns += _edge_to_anns(edge, model_element, id_to_shape_ann)
+            category = get_category(element, model_element)
+
+            # only edge type that can have another edge as src or target
+            # therefore has to be separated and moved to the end
+            if category == syntax.ASSOCIATION:
+                associations.append(element)
+                continue
+            if category in syntax.BPMNDI_SHAPE_CATEGORIES:
+                element_anns = _shape_to_anns(element, model_element, has_pools=len(collaborations) > 0)
+            else:
+                element_anns = _edge_to_anns(element, model_element, id_to_ann)
+            anns += element_anns
+            for ann in element_anns:
+                if ann.category != syntax.LABEL:
+                    id_to_ann[ann.id] = ann
+
+        for association in associations:
+            model_element = id_to_obj[association.get("bpmnElement")]
+            anns += _edge_to_anns(association, model_element, id_to_ann)
 
-        anns = shape_anns + edge_anns
         self._link_text_rel_anns(anns)
-        self._link_pools(anns)
-        self._link_lanes(anns, root)
+        if self.link_pools:
+            self._link_pools(anns)
+        if self.link_lanes:
+            self._link_lanes(anns, root)
         return anns
 
     def _link_text_rel_anns(self, anns):
@@ -169,15 +200,10 @@ class BpmnParser:
     def scale_anns_to_img_width_(self, anns: List[Annotation], bpmn_path: Path, img: Image.Image):
         img_w_annotation = parse_annotation_background_width(bpmn_path)
         scale = img.width / img_w_annotation
-        arrow_min_wh_scaled = self.arrow_min_wh * max(img.size) / self.img_max_size_ref
+
         for a in anns:
             a.bb = a.bb.scale(scale)
 
-            if a.category in syntax.BPMNDI_EDGE_CATEGORIES:
-                a.bb = a.bb.pad_min_size(
-                    w_min=arrow_min_wh_scaled, h_min=arrow_min_wh_scaled
-                )
-
             if not a.bb.is_within_img(img.width, img.height):
                 _logger.debug(
                     "%s: clipping bb %s to img (%d,%d)",
@@ -194,6 +220,14 @@ class BpmnParser:
                 a.tail = a.waypoints[0]
                 a.head = a.waypoints[-1]
 
+    def resize_arrows_to_min_wh(self, edge_anns: List[Annotation], arrow_min_wh: float):
+        for a in edge_anns:
+            assert a.category in syntax.BPMNDI_EDGE_CATEGORIES
+
+            a.bb = a.bb.pad_min_size(
+                w_min=arrow_min_wh, h_min=arrow_min_wh
+            )
+
 
 def get_ns(element: Element):
     tag_str = element.tag
@@ -295,7 +329,7 @@ def _create_id_to_obj_mapping(element):
     return id_to_obj
 
 
-def _edge_to_anns(edge: Element, model_element: Element, id_to_shape_ann: Dict[str, Annotation]):
+def _edge_to_anns(edge: Element, model_element: Element, id_to_ann: Dict[str, Annotation]):
     """
     Parses edges (see syntax.BPMNDI_EDGE_CATEGORIES)
     :param edge the BPMNDI edge element
@@ -327,10 +361,10 @@ def _edge_to_anns(edge: Element, model_element: Element, id_to_shape_ann: Dict[s
         if rel not in attrib:
             continue
         sid = attrib[rel]
-        ann = id_to_shape_ann.get(sid, None)
+        ann = id_to_ann.get(sid, None)
         if ann is None and category == syntax.ASSOCIATION:
-            # TODO implement that associations can be connected to other edges (e.g. sequenceFlow)
-            continue
+            # TODO implement that associations can be connected to other associations
+            raise InvalidBpmnException("Association has another association as src or target", sid)
         attrib[rel] = ann
     anns = [Annotation(category, bb, waypoints=waypoints, **attrib)]
 
