@@ -1,4 +1,5 @@
 import json
+import re
 from pathlib import Path
 from typing import Tuple
 
@@ -7,6 +8,16 @@ from lxml.etree import _Element as Element
 from yamlu.img import BoundingBox
 
 
+def capitalize_fc(s: str):
+    return s[0].upper() + s[1:]
+
+
+def split_camel_case(name: str) -> str:
+    # e.g. "timerStartEvent" -> "Timer Start Event"
+    words = [word.capitalize() for word in re.split(r'(?=[A-Z])', capitalize_fc(name)) if word]
+    return " ".join(words)
+
+
 def bounds_to_bb(bounds: Element) -> BoundingBox:
     xywh = {k: to_int_or_float(bounds.get(k)) for k in ["x", "y", "width", "height"]}
     bb = BoundingBox.from_xywh(**xywh, allow_neg_coord=True)
